package main

import (
	"context"
	"flag"
	"fmt"
	"log"
	"net"

	"google.golang.org/grpc"
	pb "gotokens.com/proto"
)

var (
	port = flag.Int("port", 50051, "The server port")
)

// server is used to implement helloworld.GreeterServer.
type server struct {
	manager *TokenManager
	pb.UnimplementedTokenManagerServer
}

// SayHello implements helloworld.GreeterServer
func (s *server) Create(ctx context.Context, in *pb.CreateRequest) (reply *pb.CreateReply, err error) {
	err = s.manager.Create(in.Id)
	if err != nil {
		return
	}
	return &pb.CreateReply{Message: "Create Success"}, nil
}

func (s *server) Write(ctx context.Context, in *pb.WriteRequest) (reply *pb.WriteReply, err error) {
	partial, err := s.manager.Write(in.Id, in.Name, in.Low, in.Mid, in.High)
	if err != nil {
		return
	}
	return &pb.WriteReply{Partial: partial, Message: "Write Success"}, nil
}
func (s *server) Read(ctx context.Context, in *pb.ReadRequest) (reply *pb.ReadReply, err error) {
	final, err := s.manager.Read(in.Id)
	if err != nil {
		return
	}
	return &pb.ReadReply{Final: final, Message: "Read Success"}, nil
}
func (s *server) Drop(ctx context.Context, in *pb.DropRequest) (reply *pb.DropReply, err error) {
	err = s.manager.Drop(in.Id)
	if err != nil {
		return
	}
	return &pb.DropReply{Message: "Drop Success"}, nil
}
func main() {
	flag.Parse()
	lis, err := net.Listen("tcp", fmt.Sprintf(":%d", *port))
	if err != nil {
		log.Fatalf("failed to listen: %v", err)
	}
	s := grpc.NewServer()
	manager := NewTokenManager()
	pb.RegisterTokenManagerServer(s, &server{manager: manager})
	log.Printf("server listening at %v", lis.Addr())
	if err := s.Serve(lis); err != nil {
		log.Fatalf("failed to serve: %v", err)
	}
}
